﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Web;
using System.Xml;
using System.Xml.Linq;
using Mobile.Search.Shared.Models;
using Serilog;
using ServiceStack;

namespace Mobile.Search.Providers
{
    public class BidSystemProvider
    {
        private static string Md5(string input)
        {
            // step 1, calculate MD5 hash from input
            MD5 md5 = System.Security.Cryptography.MD5.Create();
            byte[] inputBytes = System.Text.Encoding.ASCII.GetBytes(input);
            byte[] hash = md5.ComputeHash(inputBytes);

            // step 2, convert byte array to hex string
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < hash.Length; i++)
            {
                sb.Append(hash[i].ToString("X2"));
            }
            return sb.ToString();
        }

        public static string TestXml = @"<?xml version='1.0' encoding='UTF-8'?>
<request>
	<email>
		<list>{4}</list>
		<recipient>{5}</recipient>
		<domain>aol.com</domain>
		<redirect><![CDATA[http://imp.d2s.adk-mobile.com/?event=bidsystem-click&quizid={0}&questionid={1}&userid={2}&sessionid={3}&url=]]></redirect>
		<countrycode>US</countrycode>
		<subid><![CDATA[d2s]]></subid>
	</email>
</request>";

        public static IEnumerable<Listing> GetListingsFromXml(string xml)
        {
            var xmlDoc = XDocument.Parse(xml).Root;
            if (xmlDoc != null)
            {            
                var entryElement = xmlDoc.Descendants().Where(x => x.Name.LocalName == "entry");
                return xmlDoc.Elements("ResultSet")
                    .SelectMany(x => x.Elements("Listing"))
                    .Select(x => new Listing
                    {
                        Rank = 1,
                        Title = "",
                        Description =  "description",
                        SiteHost = "siteHost",
                        BiddedListing = true,
                        ResultType = "resultType",
                        SiteLanguage = "en",
                        ListingLanguage = "en",
                        PhoneNumber = "",
                        ClickUrl = "clickurl",
                        FaviconUrl = "",
                        MerchRatingNumber = "0",
                        MerchRatingUrl = "https://s3.amazonaws.com/mobile.cdn/0_stars.png",
                        //MerchRating = "",
                        //LongTitle = "",
                        ImageUrl = "",

                        // === Yahoo! API Extensions Update 2017-10-11 === //

                        // 1. Callout Extensions
                        CalloutAnnotation = "",

                        // 2. Review Extensions
                        ReviewSource = "",
                        ReviewText = "",

                        // 3. 4th Line Annotations
                        Ad4thLine = "",

                        // 4. Top Ad Annotations
                        HasTopAdAnnotation = "",
                        TopAdDomain = "0",
                        TopAdClicks = "0",

                        // 5. Security Badge
                        SecurityProvider = "",

                        // 6. Brand Annotations
                        AdOfficialBrand = "",

                        // 7. Smart Annotations
                        SmartAdAnnotation = ""
                    });

            }
            return new List<Listing>();
        }

        public static void GetListings(string sessionId, string categoryId, string ip = null)
        {
            var url = "http://bs-d2s-list-80.ak-networks.com/searches?";

            url = url.AddQueryParam("category-id", categoryId)
                .AddQueryParam("objective-id", 1)
                .AddQueryParam("count", 10)
                .AddQueryParam("default-country", "US")
                .AddQueryParam("affiliate-id", "844069")
                .AddQueryParam("exclude-central-discounting", "true")
                .AddQueryParam("mobile-listings", "true")
                .AddQueryParam("ip", ip ?? "205.232.172.74");
            var xml = url.GetXmlFromUrl(requestFilter: req =>
            {
                req.Timeout = (int)TimeSpan.FromSeconds(5).TotalMilliseconds;
            });

            GetListingsFromXml(xml);
        }

        private static Random rand = new Random();

        static Tuple<string, decimal> GetClickUrlAndBidFromXml(string xml)
        {
            var xmlDoc = XDocument.Parse(xml).Root;

            if (xmlDoc == null) return null;
            var entryElement = xmlDoc.Descendants().FirstOrDefault(x => x.Name.LocalName == "entry");

            if (entryElement == null) return null;
            var clickUrlElement = entryElement
                .Descendants().FirstOrDefault(x => x.Name.LocalName == "click-url");

            var bidElement = entryElement.Descendants().FirstOrDefault(x => x.Name.LocalName == "price-chain")
                .Descendants().FirstOrDefault();

            decimal bid = 0;
            if (bidElement != null && bidElement.Attribute("value") != null)
            {
                decimal.TryParse(bidElement.Attribute("value").Value, out bid);
            }

            return Tuple.Create(clickUrlElement.Value, bid);

        }
        public static string GetClickUrl(string clickId, string categoryId, string ip = null, string affiliateid = "844069")
        {
            try
            {
                var listingResults = new List<Tuple<string, decimal>>();

                var url = "http://bs-d2s-list-80.ak-networks.com/searches?";

                url = url.AddQueryParam("category-id", categoryId)
                    .AddQueryParam("objective-id", 1)
                    .AddQueryParam("count", 10)
                    .AddQueryParam("default-country", "US")
                    .AddQueryParam("affiliate-id", affiliateid)
                    .AddQueryParam("exclude-central-discounting", "true")
                    .AddQueryParam("mobile-listings", "true")
                    .AddQueryParam("ip", ip ?? "205.232.172.74");

                var xml = url.GetXmlFromUrl(requestFilter: req =>
                {
                    req.Timeout = (int) TimeSpan.FromSeconds(5).TotalMilliseconds;
                });

                listingResults.Add(GetClickUrlAndBidFromXml(xml));

                    //try again without mobile exclusive listings
                url = "http://bs-d2s-list-80.ak-networks.com/searches?";

                url = url.AddQueryParam("category-id", categoryId)
                    .AddQueryParam("objective-id", 1)
                    .AddQueryParam("count", 10)
                    .AddQueryParam("default-country", "US")
                    .AddQueryParam("exclude-central-discounting", "true")
                    .AddQueryParam("affiliate-id", affiliateid)
                    .AddQueryParam("ip", ip ?? "205.232.172.74");
                xml = url.GetXmlFromUrl(requestFilter: req =>
                {
                    req.Timeout = (int) TimeSpan.FromSeconds(5).TotalMilliseconds;
                });

                listingResults.Add(GetClickUrlAndBidFromXml(xml));



                var result = listingResults.Where(x => x != null).Where(x => !string.IsNullOrEmpty(x.Item1))
                    .OrderByDescending(x => x.Item2)
                    .FirstOrDefault();

                if (result == null)
                    return string.Empty;

                var clickUrl = result.Item1;
                if (clickUrl != null)
                {
                    return clickUrl + "&txn=" + HttpUtility.UrlEncode(clickId);
                }

                return string.Empty;
            }
            catch (Exception x) 
            {
                Log.Error(x, "Bid System sucks");
                return string.Empty;
            }
        }


        public static string GetResults(string quizId, string questionId, string userId, string sessionId, string listid)
        {
            var generateUser =Md5(Guid.NewGuid().ToString("N") + "@aol.com");
            var bsXml = TestXml.Fmt(quizId, questionId, userId, sessionId, listid, generateUser);
            var endpoint = "http://integrated.adstation.com/1.3/"
                .AddQueryParam("cdomain", "click.email.horoscopedaily.email")
                .AddQueryParam("idomain", "image.email.horoscopedaily.email")
                .AddQueryParam("request", bsXml)
                .AddQueryParam("nolsdb", 1)
                .AddQueryParam("token", "729fcd8c7333e1455c865b8ecbc281c6");

            var xml = endpoint.GetXmlFromUrl();
            if (xml != null)
            {
                var index = xml.IndexOf("http://imp.d2s.adk-mobile.com/?event=bidsystem-click&quizid={0}&questionid={1}&userid={2}&sessionid={3}&url=".Fmt(quizId, questionId, userId, sessionId),
                    System.StringComparison.Ordinal);

                if (index > 0)
                {
                    var firstClickUrl = xml.Substring(index, xml.IndexOf("\"", index) - index);
                    return firstClickUrl;
                }
            }

            return string.Empty;
        }
         
    }
}