﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using System.Xml.Linq;
using Mobile.Search.Shared;
using Mobile.Search.Shared.Models;
using ServiceStack;

namespace Mobile.Search.Providers
{
    public class IndeedProvider
    {
        private static readonly List<string> UserAgents = new List<string>
        {
            "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2272.101 Safari/537.36",
            "Mozilla/5.0 (Linux; U; Android 4.0.3; ko-kr; LG-L160L Build/IML74K) AppleWebkit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30"
        };

        public static SearchResult GetListings(string query, string ip, int count)
        {
            var userAgent = UserAgents[1];

            ip = ip ?? "1.2.3.4";

            var url = MakeUrl(query, ip, userAgent, count);

            var xml = GetXml(url, userAgent);

            var listings = GetListingsFromXml(xml).ToList();

            return new SearchResult
            {
                SearchId = Guid.NewGuid(),
                Ip = ip,
                UserAgent = userAgent,
                SourceTag = null,
                TypeTag = null,
                Listings = listings,
                ListingsCount = listings.Count(),
                Query = query,
                OriginalQuery = query
            };
        }

        private static string MakeUrl(string query, string ip, string userAgent, int count)
        {
            var url = "http://api.indeed.com/ads/apisearch"
                .AddQueryParam("publisher", "6782791018469226")
                .AddQueryParam("q", query)
                .AddQueryParam("sort", "relevance")
                .AddQueryParam("radius", 50)
                .AddQueryParam("limit", count)
                .AddQueryParam("format", "xml")
                .AddQueryParam("latlong", 1)
                .AddQueryParam("co", "us")
                .AddQueryParam("userip", ip)
                .AddQueryParam("useragent", userAgent)
                .AddQueryParam("v", 2);

            return url;
        }

        private static string GetXml(string url, string userAgent)
        {
            try
            {

                return url.GetStringFromUrl(
                    requestFilter: req =>
                    {
                        req.Method = "GET";
                        req.KeepAlive = false;
                        req.UserAgent = userAgent;
                        req.Headers.Add("Accept-Encoding", "gzip,deflate");
                        req.Headers.Add("Cache-Control", "no-cache");
                    });
            }
            catch (WebException webEx)
            {
                //TODO: Log
                if (webEx.IsBadRequest())
                {
                    throw new Exception(webEx.GetResponseBody(), webEx);
                }
                throw;
            }
            //return "";
        }

        public static IEnumerable<Listing> GetListingsFromXml(string xml)
        {
            var xmlDoc = XDocument.Parse(xml).Root;
            if (xmlDoc != null)
            {
                var rank = 1;
                return xmlDoc.Elements("results")
                    .SelectMany(x => x.Elements("result"))
                    .Select(x => new Listing
                    {
                        Rank = rank++,
                        Title = TryGetElementValue(x, "jobtitle"),
                        Description = TryGetElementValue(x, "snippet"),
                        SiteHost = TryGetSiteHostFromURL(x, "url"),
                        BiddedListing = true,
                        ResultType = default(string),
                        SiteLanguage = default(string),
                        ListingLanguage = default(string),
                        PhoneNumber = default(string),
                        ClickUrl = TryGetElementValue(x, "url")
                    });

            }
            return new List<Listing>();
        }

        private static string TryGetSiteHostFromURL(XElement element, string name)
        {
            var host = default(string);
            var url = element.Element("url").Value;
            if (url.Length > 0)
            {
                Uri uri = new Uri(url);
                host = uri.Host;
            }
            return host;
        }

        private static string TryGetElementValue(XElement element, string name)
        {
            var e = element.Element(name);
            return e != null ? e.Value : default(string);
        }
    }
}
