﻿using System;
using System.Linq;
using System.Text;
using Chronos;
using Chronos.Configuration;
using RabbitMQ.Client;

namespace Mobile.Search.Shared
{
    public interface IQueueHelper
    {
        void PublishMessage(string exchangeName,string routingKey, string messageType,string message);
    }

    public class QueueHelper : IQueueHelper, IDisposable
    {
        private readonly ConnectionFactory _connectionFactory;
        private IConnection _rabbitConnection;

        public QueueHelper(string rabbitConnStrKey)
        {
            var rabbitConnectionString = ConfigUtilities.GetRabbitMqConnectionString(rabbitConnStrKey);
            _connectionFactory = new ConnectionFactory
            {
                HostName = rabbitConnectionString.Host,
                Port = rabbitConnectionString.Port,
                UserName = rabbitConnectionString.Username,
                Password = rabbitConnectionString.Password,
            };

            _rabbitConnection = _connectionFactory.CreateConnection();

        }
        public void PublishMessage(string exchangeName, string routingKey, string messageType,string message)
        {
            PublishMessages(exchangeName, routingKey, messageType,new[] { message });
        }

        public void PublishMessages(string exchangeName, string routingKey, string messageType, string[] messages)
        {
            // unknown if this is needed or not, or what even happens if the connection isn't open
            if (!_rabbitConnection.IsOpen)
            {
                _rabbitConnection = _connectionFactory.CreateConnection();
            }

            using (var channel = _rabbitConnection.CreateModel())
            {
                var basicProps = channel.CreateBasicProperties();
                basicProps.ContentType = "application/json";
                basicProps.MessageId = Guid.NewGuid().ToString("D");
                basicProps.Type = messageType;
                basicProps.SetPersistent(true);

                var msgs = messages.Select(msg => Encoding.UTF8.GetBytes(msg));

                foreach (var message in msgs)
                {
                    basicProps.Timestamp = new AmqpTimestamp(DateTime.UtcNow.ToUnixTimestamp());
                    channel.BasicPublish(
                        exchange: exchangeName,
                        routingKey: routingKey,
                        basicProperties: basicProps,
                        body: message);
                }
            }
        }

        public void Dispose()
        {
            _rabbitConnection.Close();
        }
    }
}