﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Text.RegularExpressions;
using Chronos;
using Mobile.Search.Shared.Extensions;
using Mobile.Search.Shared.Models;
using Mobile.Search.Shared.Helpers;
using ServiceStack;
using ServiceStack.Text;
using System.Web;
using Serilog;
using System.Security.Cryptography;
using System.Text;
using Newtonsoft.Json.Linq;

namespace Mobile.Search.Shared
{
    public class Tracker
    {
        private static readonly QueueHelper QueueHelper = new QueueHelper("rabbitMq");
        public const string ImpressionExchange = "impressions";
        public const string BidSystemRoutingKey = "bidsystem";
        public const string SearchesRoutingKey = "preprocess";
        public const string ClicksRoutingKey = "preprocess";

        static Tracker()
        {
            JsConfig.DateHandler = DateHandler.ISO8601;
            JsConfig.PropertyConvention = PropertyConvention.Lenient;
            JsConfig.EmitCamelCaseNames = true;
        }

        public static void TrackBidSystemSearch(Dictionary<string, string> data)
        {
            QueueHelper.PublishMessage(ImpressionExchange, BidSystemRoutingKey, "bidsystem-search", data.ToJson());
        }
        public static void TrackSearch(SearchRequest request, SearchResult result, Dictionary<string, string> additionalData, bool debug)
        {
            try
            {
                var dataDictionary = new Dictionary<string, object>(StringComparer.OrdinalIgnoreCase);

                foreach (var kvp in request.ToStringDictionary())
                {
                    dataDictionary.AddOnlyIfDoesntExist(kvp.Key, kvp.Value);
                }

                foreach (var kvp in result.ToStringDictionary())
                {
                    dataDictionary.AddOrUpdate(kvp.Key, kvp.Value);
                }
                foreach (var listing in result.Listings)
                {
                    dataDictionary.AddOrUpdate("listing" + listing.Rank + "Title", listing.Title);
                    dataDictionary.AddOrUpdate("listing" + listing.Rank + "Description", listing.Description);
                    dataDictionary.AddOrUpdate("listing" + listing.Rank + "Advertiser", listing.SiteHost);
                    dataDictionary.AddOrUpdate("listing" + listing.Rank + "MerchRatingUrl", listing.MerchRatingUrl);
                    dataDictionary.AddOrUpdate("listing" + listing.Rank + "MerchRatingNumber", listing.MerchRatingNumber);
                    dataDictionary.AddOrUpdate("listing" + listing.Rank + "FaviconUrl", listing.FaviconUrl);
                    dataDictionary.AddOrUpdate("listing" + listing.Rank + "ImageUrl", listing.ImageUrl);
                    if (listing.Rank == 1)
                    {
                        dataDictionary.AddOrUpdate("listing" + listing.Rank + "ClickUrl", listing.ClickUrl);
                    }
                }

                foreach (var kvp in additionalData)
                {
                    dataDictionary.AddOrUpdate(kvp.Key, kvp.Value);
                }

                // parse additional things in the click wrapper query string
                //

                if (dataDictionary.ContainsKey("clickWrapper"))
                {
                    var clickWrapper = dataDictionary["clickWrapper"] as string;
                    if (clickWrapper != null)
                    {
                        var clickWrapperParams = GetDictionaryFromClickWrapper(clickWrapper);
                        if (clickWrapperParams["user_class"] != null)
                        {
                            dataDictionary.AddOnlyIfDoesntExist("userClass", clickWrapperParams["user_class"]);
                        }
                        if (clickWrapperParams["offer_id"] != null)
                        {
                            dataDictionary.AddOnlyIfDoesntExist("offerId", clickWrapperParams["offer_id"]);
                        }
                    }
                }

                if (!debug)
                {
                    QueueHelper.PublishMessage(ImpressionExchange, SearchesRoutingKey, "search", dataDictionary.ToJson());
                }
            }
            catch (Exception)
            {
                //TODO: Add Logging and eat exception
                // throw;
            }
        }

        public static void TrackClick(Dictionary<string, string> data)
        {
            bool is_manual_offer_blast = false;
            if (!data.ContainsKey("timestamp"))
            {
                data.Add("timestamp", DateTime.UtcNow.ToString("O"));
            }
            // this should probably be removed, it will only screw things up (4/26/16) - Kyle
            if (data.ContainsKey("type"))
            {
                var type = 0;
                int.TryParse(data["type"], out type);

                data.AddOrUpdate("sourceTag", YahooTags.GetSourceTag(type));
            }
            if (data.ContainsKey("partner"))
            {
                data.AddOrUpdate("sourceTag", data["partner"]);
            }
            if (data.ContainsKey("sourceTagOverride"))
            {
                data.AddOrUpdate("sourceTag", data["sourceTagOverride"]);

                if(data["sourceTagOverride"] == "adknowledge_mobile_2click_newsletter_search")
                {
                    is_manual_offer_blast = true;
                }
            }

            ////tracking do the same overwrite based on productid.
            ////move this to calling system later.
            if (data.ContainsKey("product") && !is_manual_offer_blast)
            {
                string product = data["product"];
                int? productId = null;
                if (!productId.HasValue && !string.IsNullOrEmpty(product))
                {
                    var p = 0;
                    if (int.TryParse(product, out p))
                    {
                        productId = p;
                    }
                }
                string sourceTagOverride = "";
                if (data.TryGetValue("sourceTagOverride", out sourceTagOverride))
                {

                    if ((string.IsNullOrEmpty(sourceTagOverride) || (!sourceTagOverride.Equals("adknowledge_mobile_2click_e_search", StringComparison.CurrentCultureIgnoreCase) 
                        && !sourceTagOverride.Equals("adknowledge_mobile_2click_d_search", StringComparison.CurrentCultureIgnoreCase)
                        && !sourceTagOverride.Equals("reachmobi_mobileinbox_Go", StringComparison.CurrentCultureIgnoreCase)
                        && !sourceTagOverride.Equals("reachmobi_mobileinbox_LearnMore", StringComparison.CurrentCultureIgnoreCase)
                        && !sourceTagOverride.Equals("reachmobi_mobileinbox_SeeIT", StringComparison.CurrentCultureIgnoreCase)
                        && !sourceTagOverride.Equals("reachmobi_mobileinbox_SeeNow", StringComparison.CurrentCultureIgnoreCase)
                        && !sourceTagOverride.Equals("reachmobi_mobileinbox_ViewNow", StringComparison.CurrentCultureIgnoreCase))))
                    {
                        //GET SOURCE TAG
                        data.AddOrUpdate("sourceTag", YahooSourceTagHelper.GetSourceTag(productId, sourceTagOverride));
                    }
                }

            }

            QueueHelper.PublishMessage(ImpressionExchange, ClicksRoutingKey, "click", data.ToJson());
        }

        public static string WrapPlainUrl(string clickWrapper, string url, Guid? searchId, Dictionary<string,string> clickWrapperReplacements = null)
        {
            if (string.IsNullOrEmpty(clickWrapper))
            {
                return url;
            }

            var parsedWrapperQuery = HttpUtility.ParseQueryString((new Uri(clickWrapper)).Query);
            var email_id = parsedWrapperQuery.Get("email_id");
            if (url.ToUpper().Contains("{EMAIL_ID}") && email_id != null)
            {
                url = Regex.Replace(url, "{EMAIL_ID}", email_id, RegexOptions.IgnoreCase);
            }
            var uuid = parsedWrapperQuery.Get("uuid");
            if (url.ToUpper().Contains("{UUID}") && uuid != null)
            {
                url = Regex.Replace(url, "{UUID}", uuid, RegexOptions.IgnoreCase);
            }
            var user_class = parsedWrapperQuery.Get("user_class");
            if (url.ToUpper().Contains("{USER_CLASS}") && user_class != null)
            {
                url = Regex.Replace(url, "{USER_CLASS}", user_class, RegexOptions.IgnoreCase);
            }
            if (url.ToUpper().Contains("{CLICK_WRAPPER}") && clickWrapper != null)
            {
                url = Regex.Replace(url, "{CLICK_WRAPPER}", HttpUtility.UrlEncode(clickWrapper), RegexOptions.IgnoreCase);
            }
            if (url.ToUpper().Contains("{DATE_STRING}"))
            {
                url = Regex.Replace(url, "{DATE_STRING}", DateTime.UtcNow.Date.ToString("yyyyMMdd"), RegexOptions.IgnoreCase);
            }

            if (searchId != null)
            {
                clickWrapper = clickWrapper.ToUpper().Contains("{SEARCH_ID}") 
                    ? Regex.Replace(clickWrapper, "{SEARCH_ID}", searchId.Value.ToString("N"), RegexOptions.IgnoreCase) 
                    : clickWrapper.AddQueryParam("searchId", searchId.Value.ToString("N"));
            }

            if (clickWrapperReplacements != null)
            {
                foreach (var kvp in clickWrapperReplacements)
                {
                    if (clickWrapper.ToUpper().Contains("{" + kvp.Key.ToUpper() + "}"))
                    {
                        clickWrapper = Regex.Replace(clickWrapper, "{" + kvp.Key + "}", HttpUtility.UrlEncode(kvp.Value),
                            RegexOptions.IgnoreCase);
                    }
                }
            }

            var wrappedUrl = Regex.Replace(clickWrapper, "{URL}", WebUtility.UrlEncode(url), RegexOptions.IgnoreCase);
            return wrappedUrl;
        }
        public static string MakeTrackerUrl(string serveUrl, string clickWrapper, Dictionary<string, string> queryParams, Listing listing, SearchResult result)
        {
            //queryParams = new Dictionary<string, string>(queryParams); // AK: why is this ???
            queryParams = new Dictionary<string, string>(queryParams, StringComparer.OrdinalIgnoreCase); // AK: prevent duplication

            // Make Cloaker Url Params
            queryParams.Remove("q");
            queryParams.Remove("clickWrapper");
            queryParams.Remove("clickwrapper"); // AK: why twice ???

            // add hardcoded params we want
            queryParams.AddOrUpdate("u", WebUtility.UrlEncode(listing.ClickUrl));
            queryParams.AddOrUpdate("searchId", result.SearchId.ToString("N"));
            queryParams.AddOrUpdate("keyword", WebUtility.UrlEncode(result.Query));
            queryParams.AddOrUpdate("listingRank", listing.Rank.ToString(CultureInfo.InvariantCulture));
            queryParams.AddOrUpdate("advertiser", WebUtility.UrlEncode(WebUtility.HtmlDecode(listing.SiteHost)));
            queryParams.AddOrUpdate("biddedListing", WebUtility.UrlEncode(listing.BiddedListing.ToString().ToLower()));
            queryParams.AddOrUpdate("listingTitle", WebUtility.UrlEncode(WebUtility.HtmlDecode(listing.Title)));
            queryParams.AddOrUpdate("listingDescription", WebUtility.UrlEncode(WebUtility.HtmlDecode(listing.Description)));
            queryParams.AddOrUpdate("related", WebUtility.UrlEncode(string.Join(",", result.RelatedSearches)));

            // add everything in the click wrapper to the click link 
            var clickWrapperParams = GetDictionaryFromClickWrapper(clickWrapper);
            foreach (var kvp in clickWrapperParams)
            {
                if (kvp.Key.ToLower() != "url")
                {
                    queryParams.AddOrUpdate(kvp.Key, kvp.Value);
                }
            }

            // add or replace the source tag with the actual one we used
            queryParams.AddOrUpdate("sourceTag", WebUtility.UrlEncode(result.SourceTag.ToLower()));

            // md5 this whenever it's in a plain url (not used for anything, could remove as of 4/29/16)
            queryParams.AddOrUpdate("encSourceTag", WebUtility.UrlEncode(result.SourceTag.ToLower().ToMd5()));

            // do this after so we are always updating with the actual type tag we use
            // (don't let the click wrapper override it)
            queryParams.AddOrUpdate("typeTag", WebUtility.UrlEncode(result.TypeTag));

            var cloakerUrl = serveUrl +
                             (serveUrl.EndsWith("?") ? queryParams.ToQueryString() : "?" + queryParams.ToQueryString());

            if (string.IsNullOrEmpty(clickWrapper))
            {
                return cloakerUrl;
            }

            var url = MakeReplacements(clickWrapper, cloakerUrl, queryParams, listing, result.SearchId.ToString("N"), result.TypeTag);

            return url;
        }


        // TODO: Make a new Tracker URL strategy for Taboola that doesn't require keyword or biddedListing
        /*public static string MakeTaboolaTrackerUrl(string serveUrl, string clickWrapper, Dictionary<string, string> queryParams, Listing listing, SearchResult result)
        {
            //queryParams = new Dictionary<string, string>(queryParams); // AK: why is this ???
            queryParams = new Dictionary<string, string>(queryParams, StringComparer.OrdinalIgnoreCase); // AK: prevent duplication

            // Make Cloaker Url Params
            queryParams.Remove("q");
            queryParams.Remove("clickWrapper");
            queryParams.Remove("clickwrapper"); // AK: why twice ???

            // add hardcoded params we want
            queryParams.AddOrUpdate("u", WebUtility.UrlEncode(listing.ClickUrl));
            queryParams.AddOrUpdate("searchId", result.SearchId.ToString("N"));
            queryParams.AddOrUpdate("keyword", WebUtility.UrlEncode(result.Query));
            queryParams.AddOrUpdate("listingRank", listing.Rank.ToString(CultureInfo.InvariantCulture));
            queryParams.AddOrUpdate("advertiser", WebUtility.UrlEncode(WebUtility.HtmlDecode(listing.SiteHost)));
            queryParams.AddOrUpdate("biddedListing", WebUtility.UrlEncode(listing.BiddedListing.ToString().ToLower()));
            queryParams.AddOrUpdate("listingTitle", WebUtility.UrlEncode(WebUtility.HtmlDecode(listing.Title)));
            queryParams.AddOrUpdate("listingDescription", WebUtility.UrlEncode(WebUtility.HtmlDecode(listing.Description)));
            // queryParams.AddOrUpdate("related", WebUtility.UrlEncode(string.Join(",", result.RelatedSearches)));

            // add everything in the click wrapper to the click link 
            var clickWrapperParams = GetDictionaryFromClickWrapper(clickWrapper);
            foreach (var kvp in clickWrapperParams)
            {
                if (kvp.Key.ToLower() != "url")
                {
                    queryParams.AddOrUpdate(kvp.Key, kvp.Value);
                }
            }

            // add or replace the source tag with the actual one we used
            // queryParams.AddOrUpdate("sourceTag", WebUtility.UrlEncode(result.SourceTag.ToLower()));

            // md5 this whenever it's in a plain url (not used for anything, could remove as of 4/29/16)
            // queryParams.AddOrUpdate("encSourceTag", WebUtility.UrlEncode(result.SourceTag.ToLower().ToMd5()));

            // do this after so we are always updating with the actual type tag we use
            // (don't let the click wrapper override it)
            // queryParams.AddOrUpdate("typeTag", WebUtility.UrlEncode(result.TypeTag));

            var cloakerUrl = serveUrl +
                             (serveUrl.EndsWith("?") ? queryParams.ToQueryString() : "?" + queryParams.ToQueryString());

            if (string.IsNullOrEmpty(clickWrapper))
            {
                return cloakerUrl;
            }

            var url = MakeReplacements(clickWrapper, cloakerUrl, queryParams, listing, result.SearchId.ToString("N"), "Taboola");

            return url;
        }*/

        private static Dictionary<string, string> GetDictionaryFromClickWrapper(string clickWrapper)
        {
            var dict = new Dictionary<string, string>();
            try
            {
                var uri = new Uri(clickWrapper);
                var query = HttpUtility.ParseQueryString(uri.Query);
                foreach (var key in query.AllKeys)
                {
                    if (dict.ContainsKey(key))
                        continue;
                    dict.Add(key, query[key]);
                }
            }
            catch
            {
                
            }

            if (dict.ContainsKey("enc"))
            {
                var enc = dict.Get("enc");
                try
                {
                    var dec = DecryptEncParams(enc);
                    var decObject = JObject.Parse(dec);
                    foreach (var i in decObject)
                    {
                        if (!dict.ContainsKey(i.Key))
                        {
                            dict.Add(i.Key, i.Value.ToString());
                        }
                    }
                }
                catch (Exception x)
                {
                    Log.Error("Error decrypting enc parameter: {}", enc);
                }
                dict.Remove("enc");
            }

            return dict;
        }

        private static bool TryExtractValueFromClickWrapper(string clickWrapper, string key, out string value)
        {
            value = string.Empty;
            if (string.IsNullOrEmpty(clickWrapper))
                return false;

            try
            {
                var uri = new Uri(clickWrapper);
                var query = HttpUtility.ParseQueryString(uri.Query);
                if (string.IsNullOrEmpty(query[key]))
                    return false;

                value = query[key];
                return true;

            }
            catch
            {
                value = string.Empty;
                return false;
            }
        }

        private static string MakeReplacements(string clickWrapper, string cloakerUrl, Dictionary<string, string> queryParams, Listing listing, string searchId, string typeTag)
        {
            var url = clickWrapper ?? "";
            var keyword = string.Empty;
            try
            {
                var usIdx = typeTag.IndexOf("_", StringComparison.Ordinal);
                if (usIdx > 0)
                {
                    keyword = typeTag.Substring(0, usIdx).Replace("+", " ");
                }
            }
            catch (Exception x)
            {
                Log.Error("Error " + x);
                keyword = typeTag;
            }
            url = Regex.Replace(url, "(?:{|%7B)URL(?:}|%7D)", WebUtility.UrlEncode(cloakerUrl), RegexOptions.IgnoreCase);
            url = Regex.Replace(url, "(?:{|%7B)ADVERTISER(?:}|%7D)", WebUtility.UrlEncode(listing.SiteHost), RegexOptions.IgnoreCase);
            url = Regex.Replace(url, "(?:{|%7B)LISTING_RANK(?:}|%7D)", WebUtility.UrlEncode(listing.Rank.ToString(CultureInfo.InvariantCulture)), RegexOptions.IgnoreCase);
            url = Regex.Replace(url, "(?:{|%7B)LISTING_DESCRIPTION(?:}|%7D)", WebUtility.UrlEncode(listing.Description), RegexOptions.IgnoreCase);
            url = Regex.Replace(url, "(?:{|%7B)LISTING_TITLE(?:}|%7D)", WebUtility.UrlEncode(listing.Description), RegexOptions.IgnoreCase);
            url = Regex.Replace(url, "(?:{|%7B)KEYWORD(?:}|%7D)", WebUtility.UrlEncode(keyword), RegexOptions.IgnoreCase);
            if (Regex.IsMatch(url,"(?:{|%7B)SEARCH_ID(?:}|%7D)", RegexOptions.IgnoreCase))
            {
                url = Regex.Replace(url, "(?:{|%7B)SEARCH_ID(?:}|%7D)", searchId, RegexOptions.IgnoreCase);
            }
            else
            {
                url = url.AddQueryParam("searchId", searchId);
            }


            foreach (var kvp in queryParams.Where(x => !string.IsNullOrEmpty(x.Key) && !string.IsNullOrEmpty(x.Value)))
            {
                url = Regex.Replace(url, "(?:{|%7B)" + kvp.Key + "(?:}%7D)", WebUtility.UrlEncode(kvp.Value), RegexOptions.IgnoreCase);
            }
            return url;
        }

        public static string DecryptEncParams(string enc)
        {
            var key = sha256_hash("secure5g;)");
            var encBytes = Base32.Decode(enc);
            var iv = encBytes.Take(16).ToArray();
            var cipher = encBytes.Skip(16).ToArray();
            var dec = RijndaelHelper.DecryptStringFromBytes(cipher, key, iv);
            return dec;
        }

        public static byte[] sha256_hash(String value)
        {
            using (SHA256 hash = SHA256Managed.Create())
            {
                Encoding enc = Encoding.UTF8;
                return hash.ComputeHash(Encoding.UTF8.GetBytes(value));
            }
        }
    }
}