﻿using System.Linq;
using Mobile.Search.Providers;
using NUnit.Framework;

namespace Mobile.Search.Test
{
    public class Tests
    {
        [Test]
        public void CanGetListingsFromXml()
        {
            var xml =
                @"<Results xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"" xsi:noNamespaceSchemaLocation=""http://dtd.overture.com/schema/affiliate/2.9/OvertureSearchResults.xsd"">
  <ResultSet id=""searchResults"" numResults=""3"" plaCount=""0"">
    <Listing rank=""1"" title=""Top 5 Cable Internet - New - Top 5 Cable Internet Picks"" description=""Compare Top Providers in Your Area!"" siteHost=""HighSpeedInternet.com/CableInternet"" biddedListing=""true"" resultType=""DB"" siteLang=""en_US"" listingLang=""en_US"">
      <ClickUrl type=""body"">http://beap.gemini.yahoo.com/cbclk?pp=n1&amp;ap=1&amp;es=kDCsQf0GIS8P1K9Ayp4Nj0xdgJY.DQaQtBOHDkcuCA2zE1luwZtdFd.hoOA8ZvKl5YY636HwIRWNMp63KT2m6cni32gEOHYx1MSBsg06TKtFdlfPNJKaalIX9IDRfkDdWd.ERDhEnrcTNXPrvO8apoC0Vfi9vC2.wXWeUORCCZ3CaeA1jTlybKkoitrgevj4Ov8uX4fGc3EMLsbsxe_1KawPZ67ogjrmngL7CbmMqW1vdS_0zKNBb67DRQhY1.Ql.165D3_DXpVrNoctARm6_o6Gg2mxAtNChYqtt8uASaUxHW2u0uxCGyiNAppKV7Gc9dIrHnaXqEk_8oXma1NpmdmM.TBwqFTAGI.RWS6i07DZz89f44exC4pbGcq6s1jBB1s4Q_uWJnCrREfp_hPKRApRxwp5Xv82hb38i0q.JTjAbmaWz.YUtlwteEeG9Usj1SrnEgO0brtvy0ZzYXjbdJ4wkod5rN0dUF0WwW8PFyEPxojeUWejBqQWK7dYWMImZhWSjlfP2XjlhHNL9X8op377xA_dAU0PgX6XcurY0AyABorBG4OqQHkvucue_.NEvVYlLiuflzZK87nR_UhsB_X2kLKhNq7bk67fN8TybJI-&amp;bv=1.0.0</ClickUrl>
      <Extensions>
        <SiteLinks>
          <SiteLink>
            <Text>Internet Availability</Text>
            <Url>
              <AppNamespace>API</AppNamespace>
              <K>10</K>
              <Url>http://beap.gemini.yahoo.com/cbclk?pp=n1&amp;ap=1&amp;es=BOYEnhoGIS.nxyT1HPF2S4LPnGGvQa_ejzkIGewM2lweelno4VDmZQKV1E1CAQvROkEnYzMFWU4c.oykZLf8sncbcTn09RRytPnsPIx4lJRE8EtWezY1uVrlgyfyIuqZefnUKEkqLPw6US0k23xVwmsBxVavBkdcDlYgd2Q3V5JYCC4DWPN4bpiEX5gmrraN5DiLWlk9rVA4Q7B0vM4BV7rdbE72xX_VSnQubpT_6.jnTOHVhSx3P4HdvvKWLGak6gsSMQfG1.uPW2mSgSlHAS00O28DCrS7emyb9m_ctPYKM12JZ9WbIqvg7ECqB.LTthlxkItf2Q9kapJKwifEUTjRj9fhzdQvE_5xAiUVgzJwm_qxF262tRgh3hBWCEAt4GGsE8Try8oRkBEbpCq1FHua2zmtVS.MXkJM6vhbrTIZQOSN7Iy8j3YENmRrO9TqwMKhjw7Ppvj1p5x5iICd1YtL16KtMLE8nPUSHHJwNyLTiEIoq7nIh_Ai9CqAf7hLzdwFKqf7pJq.Jfxc7XadwLrhRTswJPqo_nFLeGJ5M5wOK1lEYfluLnnwiOSKim_hIgayMUh0SUXnyKnzg_CkD3EsTnc-&amp;bv=1.0.0</Url>
            </Url>
          </SiteLink>
          <SiteLink>
            <Text>Bundles</Text>
            <Url>
              <AppNamespace>API</AppNamespace>
              <K>11</K>
              <Url>http://beap.gemini.yahoo.com/cbclk?pp=n1&amp;ap=1&amp;es=X0SgZekGIS_owYCfy4t._9hK9wK.u.T938qBllq3lLRUbMsreVt5ZnKxMrb8QzOtbU03J73QhvbEW488ujkmJpzkOHnl4qXySKlTwEuG0fVl93QSXWhUu7cxN5IWw.roxAdsNVKJNvdN5oEdwTdjJUdjONgzOGYKO8L4Cmwb.N9CDtUnWNIPy.vp_SysuID9go1cIA11RPv.TOzTervXnwrMb.IRD5pUAVRaTLQ6PfMxk.UFft69AyW3o2XYarjImro4wLX_ZCri8yhkp5N9epjn829LphKo5oY4bA5PhEm5Mg0G7q5gTbrYDBKe.DdLssrSkETO10evpR9nSexmB3_STYOH6a4HLy0Mec5cu1R.DlLf1ZpbY4AvNxvnz2JqzRfhrVAWgFXmj6bZ2myltcNNB2bv6040zg3QSEyRtzrONcZG7UdvgYKSzp0xaSVSmj8e6MTyHqvW.__c8o7hNfhyGKy65D2_BIbv94JF.TkJfsUFwxwLM5WCeeH.vyA_.TLXllkEOIjMh9cABaqPV1ndLTKyyrETIK_t88fP2zAZlUXw3G30GU4JkHxw_PeWOJVC84M4tnyFvxYjXmvg&amp;bv=1.0.0</Url>
            </Url>
          </SiteLink>
          <SiteLink>
            <Text>Compare Providers</Text>
            <Url>
              <AppNamespace>API</AppNamespace>
              <K>12</K>
              <Url>http://beap.gemini.yahoo.com/cbclk?pp=n1&amp;ap=1&amp;es=hLD9OrYGIS.sfNThhpqIZ4NZmGl80_0VHBEsj.GrqvHeBBsgIhkYEbeV0F34yn.x7Ab7mkQcxb3YLkCBZBYdU59ZxAinbusd4uqUutq.VqA4oYnhK06l806tSht5TPaLrC8ockEXeSCsXw_S8gLXE7Gt9u7CFT9jnr.9Ylmk5p_ee9afmSNjs59KV05Rf.XZadhzDmB3ABUAqC6JEYYJNZn4qXOEJkUlRs6v3tGBGWkeOYCRvaqA9kmAfjaebdPt5VqtRJU3hFiCJflsWCqsc80BDjicHosJFzcZpfb7MdB13hA5Z37oWOk6UMR_stwtN_JKsl0LUQbKwo7bY9juJq7Miesl1M1eZxoJu28sjNVpkgUy7qk7qw7kjrMti5M1iy3Qbf9W1Yp7xeX8Bw9D6sxohGQFMekJrRnb0ybpik8x8MgQizWpwP34rz09_F6rUEZoDfNXYrYjQZOvhf8.owIUVHK4ljbY_kUdcc7lHHSCF9aG.TqsZvdIXsOoZGwLl2IMnb.gbEFQZmz6GODmPevjO_sC4WgOSzHK0pQ1zAXYZu9Rc5SOGg6Km6G0Qjk.CJezB85H67mkZuhIeD6w0eHBWRFmcc21mw--&amp;bv=1.0.0</Url>
            </Url>
          </SiteLink>
          <SiteLink>
            <Text>Deals</Text>
            <Url>
              <AppNamespace>API</AppNamespace>
              <K>13</K>
              <Url>http://beap.gemini.yahoo.com/cbclk?pp=n1&amp;ap=1&amp;es=DoT35WoGIS8OfAUWD1h8QVooFPng_0AxQNIaAen.TjNkX5Xast51qQwws6LqTLbt8.N0Ur6881wF6oabTkQgtSWLd0Q9ZXykYwUrxEAtQjRTYGdTVRhspL5ZLHE1jsKWnMlOy0f2_A0WK0bvF7C4_eqDDv3Lh0wuUCcHouN5tnEXkrYacI5zslSYW.U0ii4y.ugeaVLK2Sua5JNXkH.rR0II2vPJcoAPSF0aD9OYQrYSxw88X83XuzV9VgPigf.L9XW57VX40SYYxPZKYvVtkd.Ra.muSSQpCxsLB4ul9auxKRvd69.zn5JodJndGHdBnnmPr4SvakKiyp4kxJYWgqeWpHoHRfOm6P6IFlYz0I9uAFB0vWt1EuEAdICl.PM_0mvDkcIRDx5Wo3mr7ILXIy7kYoIjsQtLKCwZGUiD4GdQcKZxnw4nb5NZjsFaCfbA1tWOQwtrAP04cEwCrlYmOTHwOHioTLTN_xljqnqYY_ZbRNQ4tpHAcrDXNaZKzAvBWhoIojcd0iEzD4gws8UbUrOXzlyCRxtYXsumFbU8o3yr5zELETb3scWaCBIsH.g9urAKd4AnNThQfl_RLQ--&amp;bv=1.0.0</Url>
            </Url>
          </SiteLink>
        </SiteLinks>
        <SiteLinkLayoutSuggest>1:1,2;2:3,4</SiteLinkLayoutSuggest>
      </Extensions>
    </Listing>
    <Listing rank=""2"" title=""DISH® TV + Internet = $35"" description=""$19.99 TV + $14.95 Internet - Your #1 TV Service Provider"" siteHost=""PlanetDISH.com/DISH-Deals"" biddedListing=""true"" resultType=""Broad"" siteLang=""en_US"" listingLang=""en_US"" phoneNumber=""1-855-978-3474"">
      <ClickUrl type=""body"">http://40000170.r.msn.com/?ld=d30i7M55Mo6l5888s855xoKTVUCUw59sA9oX8PVfIiGAZQmO7AHZcAvRK2JWI5YgLvzVlEsgFUGOxgFjbOqlXFNyeMMmZZ8NHPL0xic8iPslRXqXbYJl7nH-q1UkgO88ejaRWeY-h5fOYxEDi_Hv70aEyaE3o&amp;u=https%3a%2f%2fplanetdish.com%2fdish-deals%2f%3futm_source%3dbing%26utm_medium%3dpd-dip-cpc%26utm_campaign%3dbing-ppc-dip-key-terms-tv-providers</ClickUrl>
      <Extensions>
        <SiteLinks>
          <SiteLink>
            <Text>DISH Vs. The Competition</Text>
            <Url>
              <AppNamespace>API</AppNamespace>
              <K>5042</K>
              <Url>http://40000170.r.msn.com/?ld=d3dx-F4Z3Xuwn1C7PEjRPtEjVUCUyqZ_-VP6oNSHkoLLztqzbtbQYfO2njIZIPOGHekUVNNQCTAiWPY__kVQO8gzuuvjxGoHK7E30ARwFbhuqA4G-PlsoSsN8UKZvmfuF86mD51r6r_34Zq2yZIPQvC0p6rns&amp;u=planetdish.com%2fdish-vs-directv%2f</Url>
            </Url>
          </SiteLink>
          <SiteLink>
            <Text>Join Without A Contract</Text>
            <Url>
              <AppNamespace>API</AppNamespace>
              <K>5044</K>
              <Url>http://40000170.r.msn.com/?ld=d3Vgf90820Tf5TfQgwxdpu3zVUCUxXCqcS7Ulto9HIPqhzzuRWtPHwUij4SAi4ReERKzuXDVMmVbKeBvZNfqppj4RhiL2elP06DHHNRLKV3xfLtDul6A5d6VsMzjrThStgw7ItVnAMJ-96N_8SOUUdQXx7u5I&amp;u=planetdish.com%2fflex-tv%2f</Url>
            </Url>
          </SiteLink>
        </SiteLinks>
        <SiteLinkLayoutSuggest>1:1,2</SiteLinkLayoutSuggest>
      </Extensions>
    </Listing>
    <Listing rank=""3"" title=""$14.95 Cable Provider"" description=""Compare Internet Providers - Increase Your Internet Speed For Less!"" siteHost=""InternetNearMe.com/CableInternet"" biddedListing=""true"" resultType=""Exact"" siteLang=""en_US"" listingLang=""en_US"" phoneNumber="""">
      <ClickUrl type=""body"">http://40032899.r.msn.com/?ld=d3BDMWrPdUtUA97l-DtPyvvzVUCUxvwgvEnu2hihLaLuDZ84qxBxSUo-qFujnZIlpYnb9LoFpfVPLUKslfCCDKrRd1g2BOpOLh16e69tEHHZkw0K0M2q-P49r9jEr5IRpg-kNgoGowZ63XcdsbDjRhfMANw_w&amp;u=InternetNearMe.com</ClickUrl>
    </Listing>
    <NextArgs>Keywords=cable%20internet%20service%20provider&amp;xargs=12KPjg1oZpy5a3vOHvKvjFT%5FXBhg9O0JCw5Is%5FWMQaRp8L%5FXd7EbEoO92ExpctD%2DFmzz7g&amp;hData=12KPjg1o1gls79vr2vdLnITuOGxl0EnJC0%2D8kfDJR5GtVc8HZpUuh3T5Px</NextArgs>
  </ResultSet>
  <NextArgs>Keywords=cable%20internet%20service%20provider&amp;xargs=12KPjg1oZpy5a3vOHvKvjFT%5FXBhg9O0JCw5Is%5FWMQaRp8L%5FXd7EbEoO92ExpctD%2DFmzz7g&amp;hData=12KPjg1o1gls79vr2vdLnITuOGxl0EnJC0%2D8kfDJR5GtVc8HZpUuh3T5Px</NextArgs>
</Results>";

            var listings = YahooProvider.GetListingsFromXml(xml)
                .ToList();

            Assert.NotNull(listings);
        }

    }
   
}
