﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Globalization;
using System.IdentityModel.Claims;
using System.Linq;
using System.Threading.Tasks;
using System.Web;
using Microsoft.Owin.Security;
using Microsoft.Owin.Security.Cookies;
using Microsoft.Owin.Security.OpenIdConnect;
using Microsoft.IdentityModel.Clients.ActiveDirectory;
using Owin;
using Microsoft.IdentityModel.Protocols;

namespace Mobile.Search.Web
{
    public partial class Startup
    {
        private static string clientId = ConfigurationManager.AppSettings["ida:ClientId"];
            private static string appKey = ConfigurationManager.AppSettings["ida:ClientSecret"];
            private static string aadInstance = ConfigurationManager.AppSettings["ida:AADInstance"];
            private static string tenantId = ConfigurationManager.AppSettings["ida:TenantId"];
            private static string postLogoutRedirectUri = ConfigurationManager.AppSettings["ida:PostLogoutRedirectUri"];

            public static readonly string Authority = aadInstance + tenantId;

            public void ConfigureAuth(IAppBuilder app)
            {


                app.SetDefaultSignInAsAuthenticationType(CookieAuthenticationDefaults.AuthenticationType);

                app.UseCookieAuthentication(new CookieAuthenticationOptions());


                var auth = new OpenIdConnectAuthenticationOptions
                {
                    ClientId = clientId,
                    Authority = Authority,
                    PostLogoutRedirectUri = postLogoutRedirectUri,
                    RedirectUri = postLogoutRedirectUri,

                    ProtocolValidator = new OpenIdConnectProtocolValidator() { NonceLifetime = new TimeSpan(1, 0, 0, 0) },


                    Notifications = new OpenIdConnectAuthenticationNotifications()
                    {
                        AuthenticationFailed = context =>
                        {
                            if (context.Exception is OpenIdConnectProtocolInvalidNonceException && context.Exception.Message.Contains("IDX10316"))
                            {

                                context.HandleResponse();
                                // Redirect to the originally requested URL
                                context.Response.Redirect(context.Request.Uri.PathAndQuery);
                            }

                            context.HandleResponse();
                            context.Response.Redirect("/RaiseError?message=" + context.Exception.Message);
                            return Task.FromResult(0);
                        }
                    }
                };

                app.UseOpenIdConnectAuthentication(auth);


            }
        }
    }