﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using Amazon;
using Amazon.Runtime;
using Chronos.AWS;
using Chronos.Configuration;
using Mobile.Search.Providers;
using Mobile.Search.Shared;
using Mobile.Search.Shared.Models;
using Mobile.Search.Web.Extensions;
using Mobile.Search.Web.Helpers;
using Mobile.Search.Web.Models;
using Newtonsoft.Json.Linq;

namespace Mobile.Search.Web.Controllers
{
    public class KeywordBlockController : Controller
    {
        private readonly Dynamo _dynamoClient;
        private static readonly string AwsAccessKey = ConfigUtilities.GetAppSetting("AWSAccessKey");
        private static readonly string AwsSecretKey = ConfigUtilities.GetAppSetting("AWSSecretKey");
        private static readonly IUrlHelper UrlHelper = new SearchUrlHelper();
        public KeywordBlockController()
        {
            _dynamoClient = new Dynamo(new BasicAWSCredentials(AwsAccessKey, AwsSecretKey), RegionEndpoint.USWest2);
        }

        public ActionResult Results(string templateKey,string sourceTag, string typeTag, string query, int resultsCount = 5,string ip = null, string userAgent = null, string clickWrapper = null)
        {
            var searchRequest = new SearchRequest
            {
                BiddedResults = resultsCount,
                Ip = ip ?? RequestHelper.GetIp(HttpContext),
                NonBiddedResults = 0,
                Query = query,
                ServeUrl = UrlHelper.GetAuthority(Request) + "/search",
                SourceTag = sourceTag,
                TypeTag = typeTag,
                UseFakeUserAgent = false,
                UserAgent = userAgent ?? Request.UserAgent
            };
            var searchResult = YahooProvider.GetListings(searchRequest);
            searchResult.ClickUrlFunc = listing => Tracker.MakeTrackerUrl(searchRequest.ServeUrl, clickWrapper ?? string.Empty, new Dictionary<string, string>(), listing, searchResult);

            var listings = new List<Listing>();
            foreach (var listing in searchResult.Listings)
            {
                listing.ClickUrl = searchResult.ClickUrlFunc(listing);
                listings.Add(listing);
            }


            var json = _dynamoClient.Get("prd_2click_templates", "templateKey", templateKey);
            if (string.IsNullOrEmpty(json))
            {
                return Json(new
                {
                    Status = "Error",
                    Message = "Couldn't find a template with the key: " + templateKey
                }, JsonRequestBehavior.AllowGet);
            }

            var jsonObject = JObject.Parse(json);
            var html = (string)jsonObject["html"];
            if (html != null)
            {
                var templateVars = new TemplateVariables
                {
                    Keyword = query
                };
                PopulateTemplateVarsWithListings(searchResult.Listings.ToList(), searchResult.ClickUrlFunc, ref templateVars);
                var renderedTemplate = Nustache.Core.Render.StringToString(html, templateVars);

                return Content(renderedTemplate);
            }

            return HttpNotFound("Invalid Template");
        }

        private void PopulateTemplateVarsWithListings(List<Listing> listings, Func<Listing, string> clickUrlFunc, ref TemplateVariables templateVars)
        {
            if (listings.Count > 0)
            {
                templateVars.Listing1Title = listings[0].Title;
                templateVars.Listing1Advertiser = listings[0].SiteHost;
                templateVars.Listing1Description = listings[0].Description;
                templateVars.Listing1ClickUrl = clickUrlFunc(listings[0]);
            }
            if (listings.Count > 1)
            {
                templateVars.Listing2Title = listings[1].Title;
                templateVars.Listing2Advertiser = listings[1].SiteHost;
                templateVars.Listing2Description = listings[1].Description;
                templateVars.Listing2ClickUrl = clickUrlFunc(listings[1]);
            }
            if (listings.Count > 2)
            {
                templateVars.Listing3Title = listings[2].Title;
                templateVars.Listing3Advertiser = listings[2].SiteHost;
                templateVars.Listing3Description = listings[2].Description;
                templateVars.Listing3ClickUrl = clickUrlFunc(listings[2]);
            }
            if (listings.Count > 3)
            {
                templateVars.Listing4Title = listings[3].Title;
                templateVars.Listing4Advertiser = listings[3].SiteHost;
                templateVars.Listing4Description = listings[3].Description;
                templateVars.Listing4ClickUrl = clickUrlFunc(listings[3]);
            }
            if (listings.Count > 4)
            {
                templateVars.Listing5Title = listings[4].Title;
                templateVars.Listing5Advertiser = listings[4].SiteHost;
                templateVars.Listing5Description = listings[4].Description;
                templateVars.Listing5ClickUrl = clickUrlFunc(listings[4]);
            }
        }
    }
}