﻿using System;
using System.Collections.Generic;
using System.Linq;
using Serilog;
using System.Web.Mvc;
using Amazon;
using Amazon.DynamoDBv2.DocumentModel;
using Amazon.Runtime;
using Chronos.AWS;
using Chronos.Configuration;
using Newtonsoft.Json.Linq;
using System.Web;
using Amazon.S3;
using Amazon.S3.Model;
using System.IO;
using System.Collections;
using Newtonsoft.Json;
using System.Net.Mail;

using System.Text.RegularExpressions;
using System.Text;

namespace Mobile.Search.Web.Controllers
{
    [Authorize]
    public class TemplateController : Controller
    {
        public AmazonS3Client S3Client { get; set; }
        private readonly Dynamo _dynamoClient;
        private static readonly string AwsAccessKey = ConfigUtilities.GetAppSetting("AWSAccessKey");
        private static readonly string AwsSecretKey = ConfigUtilities.GetAppSetting("AWSSecretKey");
        private static readonly string SmtpMinboxHost = ConfigUtilities.GetAppSetting("SMTPMinboxHost");
        private static readonly string SmtpSendGridHost = ConfigUtilities.GetAppSetting("SMTPSendGridHost");
        private static readonly string SmtpSendGridUsername = ConfigUtilities.GetAppSetting("SMTPSendGridUser");
        private static readonly string SmtpSendGridPassword = ConfigUtilities.GetAppSetting("SMTPSendGridPass");

        private static readonly int Port = int.Parse(ConfigUtilities.GetAppSetting("Port"));
        private static readonly string FromEmail = ConfigUtilities.GetAppSetting("FromMailAddress");


        public TemplateController()
        {
            _dynamoClient = new Dynamo(new BasicAWSCredentials(AwsAccessKey, AwsSecretKey), RegionEndpoint.USWest2);
        }
        // GET: Template
        public ActionResult Index()
        {
            var tbl = Table.LoadTable(_dynamoClient.AmazonDynamoDbClient, "prd_2click_templates");
            var scanFilter = new ScanFilter();
            scanFilter.AddCondition("deleted", ScanOperator.NotEqual, new DynamoDBBool(true));
            var search = tbl.Scan(scanFilter);

            var documentList = new List<Document>();
            do
            {
                documentList.AddRange(search.GetNextSet());
            } while (!search.IsDone);


            var templateDict = new Dictionary<string, string>();
            foreach (var document in documentList.Where(x => x.ContainsKey("templateKey")))
            {
                var value = default(DynamoDBEntry);
                var html = string.Empty;
                if (document.TryGetValue("html", out value))
                {
                    html = value.AsString();
                }
                templateDict.Add(document["templateKey"].AsString(), html);
            }

            // sort dictionary
            templateDict = templateDict.OrderBy(x => x.Key).ToDictionary(x => x.Key, x => x.Value);
            return View(templateDict);
        }

        // GET: Template/Create
        public ActionResult Create()
        {
            return View("Edit", new KeyValuePair<string, string>(Chronos.GenerateRandom.String(6), "<html></html>"));
        }


        public ActionResult CreateFroala()
        {
            return View("FroalaIndex", new KeyValuePair<string, string>(Chronos.GenerateRandom.String(6), "<html></html>"));
        }
        public ActionResult S3Signature()
        {
            FroalaEditor.S3Config config = new FroalaEditor.S3Config
            {
                Bucket = "mobileinbox-image",
                Region = "us-west-2",
                KeyStart = "Commons/",
                Acl = "public-read",
                // AWS keys.
                AccessKey = AwsAccessKey,
                SecretKey = AwsSecretKey
            };

            return Json(FroalaEditor.S3.GetHash(config), JsonRequestBehavior.AllowGet);
        }


        // GET: Template/Edit/5
        public ActionResult Edit(string templateKey)
        {
            var json = _dynamoClient.Get("prd_2click_templates", "templateKey", templateKey);
            var j = JObject.Parse(json);
            var kvp = new KeyValuePair<string, string>(((string)j["templateKey"]).Trim(), (string)j["html"]);

            return View("FroalaIndex", kvp);
        }


        // POST: Template/Edit/5
        [HttpPost, ValidateInput(false)]
        public ActionResult Edit(string key, string value)
        {
            try
            {
                var d = new Dictionary<string, object>
                {
                    {"templateKey", key.ToLower()},
                    {"html", value}
                };
                _dynamoClient.WriteSingle("prd_2click_templates", d);

                return RedirectToAction("Index");
            }
            catch (Exception x)
            {
                Log.Error("Edit: Error " + x);
                return View();
            }
        }

        public ActionResult FroalaIndex(string templateKey)
        {
            var json = _dynamoClient.Get("prd_2click_templates", "templateKey", templateKey);
            var j = JObject.Parse(json);
            var kvp = new KeyValuePair<string, string>((string)j["templateKey"], (string)j["html"]);

            return View(kvp);
        }


        // POST: Template/Edit/5
        [ValidateInput(false)]
        [HttpPost]
        public ActionResult FroalaIndex(string key, string value, string EmailId, bool isSendEmail = false, bool WrapUrl = false, bool AdvertiserHasUnSub = false, bool isGetUrl = false)
        {
            try
            {

                string url = "http://config.minbox.email/Offer/Create?template_attributes=";
                string OriginalValue = value;
                Match m;
                string HRefPattern = "href\\s*=\\s*(?:[\"'](?<1>[^\"']*)[\"']|(?<1>\\S+))";
                //  _dynamoClient.WriteSingle("prd_2click_templates", d);

                m = Regex.Match(value, HRefPattern,
                                RegexOptions.IgnoreCase | RegexOptions.Compiled,
                                TimeSpan.FromSeconds(1));
                int count = Regex.Matches(value, HRefPattern).Count;

                //if (!isSendEmail)
                //    WrapUrl = true;
                HashSet<string> Url = new HashSet<string>();
                HashSet<string> UrlWrap = new HashSet<string>();
                StringBuilder sb = new StringBuilder();

                while (m.Success && WrapUrl)
                {
                    count = count - 1;
                    if (!m.Value.Contains("Wrap"))
                    {
                        string to_replace = String.Format("{{{{WrapUrl('{0}')}}}}", m.Groups[1]);
                        if (count == 0 && AdvertiserHasUnSub)
                        {
                            to_replace = String.Format("{{{{WrapAdvertiserUnsubscribeUrl('{0}')}}}}", m.Groups[1]);
                        }
                        if (m.Groups[1].Value.Count() > 0 && !m.Groups[1].Value.Contains("anchor") && !Url.Contains(m.Groups[1].Value) && !m.Groups[1].Value.Contains(".css") && !m.Groups[1].Value.Contains(".js"))
                        {
                            string changeset = Request.Form[count.ToString()];
                            if (changeset != null)
                            {
                                to_replace = changeset;
                            }

                            value = value.Replace(m.Groups[1].Value, to_replace);
                            Url.Add(m.Groups[1].Value);
                            UrlWrap.Add(to_replace);
                            string s = String.Format("<textarea id='{0}' name='{0}' runat='server' style =width:90% rows=1 cols=100>{1}</textarea>", count, to_replace);
                            sb.Append(s);
                            sb.Append("<br/> <br/>");
                        }
                    }
                    m = m.NextMatch();
                }
                var d = new Dictionary<string, object>
                {
                    {"templateKey", key.ToLower()},
                    {"html", value}
                };

                if (isGetUrl)
                {

                    return Json(new { Allstring = sb.ToString() });

                }
                else if (isSendEmail)
                {
                    if (sendEmail(value, EmailId))
                    {
                        return Json(new { status = "Success" });
                    }

                    else
                    {
                        return Json(new { status = "Failed" });
                    }

                }

                else
                {
                    _dynamoClient.WriteSingle("prd_2click_templates", d);
                }
                url = url + key;


                return Redirect(url);
            }
            catch (Exception x)
            {
                Log.Error("Edit: Error " + x);
                return Json(new { status = "Failed" });

            }
        }

        public ActionResult SendEmail(string value, string EmailId)
        {

            return View();

        }
        public bool sendEmail(string value, string email_id)
        {
            MailAddress from = new MailAddress(FromEmail, "TemplateTest");
            MailAddress to = new MailAddress(email_id);
            MailMessage message = new MailMessage(from, to);
            message.Subject = "Template Testing Via Html Editor";
            message.Body = value;
            message.IsBodyHtml = true;
            SmtpClient client = new SmtpClient();
            client.Port = Port;


            if (email_id.Contains("minbox"))
            {
                client.Host = SmtpMinboxHost;

            }
            else
            {
                client.Host = SmtpSendGridHost;
                System.Net.NetworkCredential credentials = new System.Net.NetworkCredential(SmtpSendGridUsername, SmtpSendGridPassword);
                client.Credentials = credentials;
            }
            // client.Port = Port;
            try
            {
                client.Send(message);
                return true;

            }

            catch (Exception ex)
            {
                Console.WriteLine("Exception caught in CreateTestMessage4(): {0}",
                            ex.ToString());
                return false;

            }
        }


        // GET: Template/Delete/5
        public ActionResult Delete(string templateKey)
        {
            _dynamoClient.WriteSingle("prd_2click_templates", new Dictionary<string, object>
            {
                {"templateKey",templateKey.ToLower()},
                {"deleted", true}
            });
            return RedirectToAction("Index");
        }

        public ActionResult AdvertiserBlacklist(String keyword = null, String advertiserUrl = null, bool add = false, bool remove = false)
        {
            if (add && keyword != null && advertiserUrl != null)
            {
                _dynamoClient.WriteSingle("prd_2click_blacklist2", new Dictionary<string, object>
                {
                    {"keyword", keyword.ToLower()},
                    {"advertiserUrl", advertiserUrl.ToLower()},
                });
                return RedirectToAction("AdvertiserBlacklist");
            }
            else if (remove && advertiserUrl != null)
            {
                _dynamoClient.DeleteSingle("prd_2click_blacklist2", new Dictionary<string, object>
                {
                    {"keyword", keyword.ToLower()},
                    {"advertiserUrl", advertiserUrl.ToLower()},
                });
                return RedirectToAction("AdvertiserBlacklist");
            }

            return View(GetAdvertiserBlacklist());
        }

        public static List<Dictionary<string, string>> GetAdvertiserBlacklist()
        {
            var _dynamoClient = new Dynamo(new BasicAWSCredentials(ConfigUtilities.GetAppSetting("AWSAccessKey"), ConfigUtilities.GetAppSetting("AWSSecretKey")), RegionEndpoint.USWest2);

            var jsonList = _dynamoClient.GetAll("prd_2click_blacklist2");
            var blacklist = jsonList.Select(x =>
            {
                var jObj = JObject.Parse(x);
                return new Dictionary<string, string>()
                {
                    {"keyword", (string)jObj["keyword"]},
                    {"advertiserUrl", (string)jObj["advertiserUrl"]},
                };
            }).ToList();
            return blacklist;
        }
        public ActionResult UploadFiles()
        {
            if (Request.Files["file"] != null)
            {
                HttpPostedFileBase MyFile = Request.Files["file"];

                // Setting location to upload files
                string TargetLocation = Server.MapPath("~/Files/");

                try
                {
                    if (MyFile.ContentLength > 0)
                    {
                        // Get File Extension
                        string Extension = Path.GetExtension(MyFile.FileName);

                        // Determining file name. You can format it as you wish.
                        string FileName = Path.GetFileName(MyFile.FileName);
                        FileName = Guid.NewGuid().ToString().Substring(0, 8);

                        // Determining file size.
                        int FileSize = MyFile.ContentLength;

                        // Creating a byte array corresponding to file size.
                        byte[] FileByteArray = new byte[FileSize];

                        // Basic validation for file extension
                        string[] AllowedExtension = { ".gif", ".jpeg", ".jpg", ".png", ".svg", ".blob" };
                        string[] AllowedMimeType = { "image/gif", "image/jpeg", "image/pjpeg", "image/x-png", "image/png", "image/svg+xml" };

                        if (AllowedExtension.Contains(Extension) && AllowedMimeType.Contains(MimeMapping.GetMimeMapping(MyFile.FileName)))
                        {
                            // Posted file is being pushed into byte array.
                            MyFile.InputStream.Read(FileByteArray, 0, FileSize);

                            // Uploading properly formatted file to server.
                            MyFile.SaveAs(TargetLocation + FileName + Extension);
                            string json = "";
                            Hashtable resp = new Hashtable();
                            string urlPath = MapURL(TargetLocation) + FileName + Extension;

                            // Make the response an json object
                            resp.Add("link", urlPath);
                            json = JsonConvert.SerializeObject(resp);

                            // Clear and send the response back to the browser.
                            Response.Clear();
                            Response.ContentType = "application/json; charset=utf-8";
                            Response.Write(json);
                            Response.End();
                        }
                        else
                        {
                            // Handle validation errors
                        }
                    }
                }

                catch (Exception ex)
                {
                    // Handle errors
                }
            }
            return View();
        }

        // Convert file path to url
        // http://stackoverflow.com/questions/16007/how-do-i-convert-a-file-path-to-a-url-in-asp-net
        private string MapURL(string path)
        {
            string appPath = Server.MapPath("/").ToLower();
            return string.Format("/{0}", path.ToLower().Replace(appPath, "").Replace(@"\", "/"));
        }
    }
}
