﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Mvc;
using Mobile.Search.Providers;
using Mobile.Search.Shared;
using Mobile.Search.Shared.Models;
using Mobile.Search.Web.Extensions;
using Mobile.Search.Web.Helpers;
using ServiceStack;
using Serilog;

namespace Mobile.Search.Web.Controllers
{
    public class TestController : Controller
    {
        private static readonly IUrlHelper urlHelper = new SearchUrlHelper();
        // GET: Test
        public ActionResult CssTarget()
        {
            return View();
        }

        public ActionResult Debt(string quizId, string questionId, string userId, string sessionId)
        {
            return Redirect(BidSystemProvider.GetResults(quizId, questionId, userId, sessionId, "24044"));
        }

        public ActionResult BidSystemClick(string quizId, string questionId, string userId, string sessionId,
            string listid)
        {
            return Redirect(BidSystemProvider.GetResults(quizId, questionId, userId, sessionId, listid));
        }
        public ActionResult TwoClickCustom(string keyword, string clickWrapper = null, string firstClickWrapper = null,
            string footer = null, string templateKey = null)
        {
             clickWrapper = clickWrapper ?? "";
            if (!footer.IsNullOrEmpty())
            {
                try
                {
                    ViewBag.FooterHtml = Encoding.UTF8.GetString(Convert.FromBase64String(footer));
                }
                catch (Exception x)
                {
                    Log.Error("Error " + x);
                }
            }
            var req = new SearchRequest
            {
                BiddedResults = 5,
                NonBiddedResults = 0,
                Ip = RequestHelper.GetIp(HttpContext),
                Query = HttpUtility.UrlDecode(keyword),
                ServeUrl = urlHelper.GetAuthority(Request) + "/search",
                SourceTagId = 1,
                TypeTag = keyword,
                UserAgent = Request.UserAgent,
                Timestamp = DateTime.UtcNow,
                // Always fake user agent for now
                UseFakeUserAgent = true
            };
            var firstResult = YahooProvider.GetListings(req).Listings.FirstOrDefault();
            var clickUrl = "/search/template?keyword=" + WebUtility.UrlEncode(keyword) +
                           "&templateKey=" + WebUtility.UrlEncode(templateKey) + "&subid=customtemplate&clickWrapper=" + WebUtility.UrlEncode(clickWrapper);

            //firstClickWrapper = firstClickWrapper;
            var result = new SearchResult
            {
                UserAgent = Request.UserAgent,
                ClickUrlFunc = x => firstClickWrapper != null 
                    ? Regex.Replace(firstClickWrapper, "{URL}", WebUtility.UrlEncode(urlHelper.GetAuthority(Request) + x.ClickUrl))
                    : clickUrl,
                Ip = RequestHelper.GetIp(HttpContext),
                Listings = new List<Listing>
                {
                    new Listing
                    {
                        BiddedListing = true,
                        ClickUrl = clickUrl,
                        Description = (firstResult != null) ?  firstResult.Description : "No results could be found",
                        Rank = 1,
                        SiteHost = (firstResult != null) ? firstResult.SiteHost : "",
                        Title = (firstResult != null) ? firstResult.Title : "",
                    }
                },
                ListingsCount = 1,
            };
            return View("GoogleSingle", result);           
        }
        public ActionResult TwoClickYahoo(string keyword, string clickWrapper = null, string firstClickWrapper = null, string footer = null)
        {
            clickWrapper = clickWrapper ?? "";
            if (!footer.IsNullOrEmpty())
            {
                try
                {
                    ViewBag.FooterHtml = Encoding.UTF8.GetString(Convert.FromBase64String(footer));
                }
                catch (Exception x)
                {
                    Log.Error("Error " + x);
                }
            }
            var req = new SearchRequest
            {
                BiddedResults = 5,
                NonBiddedResults = 0,
                Ip = RequestHelper.GetIp(HttpContext),
                Query = HttpUtility.UrlDecode(keyword),
                ServeUrl = urlHelper.GetAuthority(Request) + "/search",
                SourceTagId = 1,
                TypeTag = keyword,
                UserAgent = Request.UserAgent,
                Timestamp = DateTime.UtcNow,
                // Always fake user agent for now
                UseFakeUserAgent = true
            };
            var firstResult = YahooProvider.GetListings(req).Listings.FirstOrDefault();
            var clickUrl = "/search/frame?keyword=" + WebUtility.UrlEncode(keyword) +
                           "&layout=google&subid=legal_2click&type=1&clickWrapper=" + WebUtility.UrlEncode(clickWrapper);

            //firstClickWrapper = firstClickWrapper;
            var result = new SearchResult
            {
                UserAgent = Request.UserAgent,
                ClickUrlFunc = x => firstClickWrapper != null 
                    ? Regex.Replace(firstClickWrapper, "{URL}", WebUtility.UrlEncode(urlHelper.GetAuthority(Request) + x.ClickUrl))
                    : clickUrl,
                Ip = RequestHelper.GetIp(HttpContext),
                Listings = new List<Listing>
                {
                    new Listing
                    {
                        BiddedListing = true,
                        ClickUrl = "/search/frame?keyword=" + WebUtility.UrlEncode(keyword) + "&layout=google&subid=legal_2click&type=1&clickWrapper=" + WebUtility.UrlEncode(clickWrapper),
                        Description = (firstResult != null) ?  firstResult.Description : "No results could be found",
                        Rank = 1,
                        SiteHost = (firstResult != null) ? firstResult.SiteHost : "",
                        Title = (firstResult != null) ? firstResult.Title : "",
                    }
                },
                ListingsCount = 1,
            };
            return View("GoogleSingle", result);
        }

        public ActionResult Legal(string clickWrapper = null,string firstClickWrapper = null, string footer = null)
        {
            clickWrapper = clickWrapper ?? "";
            if (!footer.IsNullOrEmpty())
            {
                try
                {
                    ViewBag.FooterHtml = Encoding.UTF8.GetString(Convert.FromBase64String(footer));
                }
                catch (Exception x)
                {
                    Log.Error("Error " + x);
                }
            }
            var result = new SearchResult
            {
                UserAgent = Request.UserAgent,
                ClickUrlFunc = x => Regex.Replace(firstClickWrapper, "{URL}", WebUtility.UrlEncode(urlHelper.GetAuthority(Request) + x.ClickUrl)),
                Ip = RequestHelper.GetIp(HttpContext),
                Listings = new List<Listing>
                {
                    new Listing
                    {
                        BiddedListing = true,
                        ClickUrl = "/search/frame?keyword=personal%2Binjury%2Blawyer&layout=google&subid=legal_2click&type=1&clickWrapper=" + WebUtility.UrlEncode(clickWrapper),
                        Description = "Click and explore your options",
                        Rank = 1,
                        SiteHost = "Seeking Legal Advice?",
                        Title = "Get Help Finding An Attorney!",
                    }
                },
                ListingsCount = 1,
            };
            var secondClick = result.ClickUrlFunc(result.Listings.First());
            return View("GoogleSingle", result);
        }
        public ActionResult CheckboxHack()
        {
            return View();
        }
    }
}