﻿using System;
using System.Collections.Generic;
using System.Linq;
using Serilog;
using Amazon;
using Amazon.Runtime;
using Chronos.Configuration;
using System.IO;
using System.IO.Compression;
using Amazon.DynamoDBv2.Model;
using Amazon.DynamoDBv2;

namespace Mobile.Search.Web.Helpers
{
    public class ClickUrlStorage
    {
        private static readonly string AWSAccessKey = ConfigUtilities.GetAppSetting("AWSAccessKey");
        private static readonly string AWSSecretKey = ConfigUtilities.GetAppSetting("AWSSecretKey");

        private static readonly string TableName = "prd_click_urls";

        public static string TryGetClickUrlFromDynamo(string emailId, string key)
        {
            try
            {
                var dynamoClient = new AmazonDynamoDBClient(new BasicAWSCredentials(AWSAccessKey, AWSSecretKey), RegionEndpoint.USWest2);
                var request = new QueryRequest
                {
                    TableName = TableName,
                    KeyConditions = new Dictionary<string, Condition>
                    {
                        {
                           "emailId",
                           new Condition
                           {
                               ComparisonOperator = "EQ",
                               AttributeValueList = new List<AttributeValue>
                               {
                                   new AttributeValue { S = emailId }
                               }
                           }
                        }
                    },
                };
                var resp = dynamoClient.Query(request);
                var item = resp.Items.FirstOrDefault();
                if (item != null)
                {
                    var attr = item[key];
                    if (attr.S != null)
                    {
                        return attr.S;
                    }
                    else
                    {
                        return FromGzipMemoryStream(attr.B);
                    }
                }
            }
            catch (Exception x)
            {
                Log.Error("Error " + x);
            }
            return string.Empty;
        }

        public static bool SaveClickUrlsToDynamo(string emailId, Dictionary<string, string> urls)
        {
            try
            {
                var dynamoClient = new AmazonDynamoDBClient(new BasicAWSCredentials(AWSAccessKey, AWSSecretKey), RegionEndpoint.USWest2);
                var row = new Dictionary<string, AttributeValue>()
                {
                    { "emailId", new AttributeValue { S = emailId } },
                    { "timestamp", new AttributeValue { S = DateTime.UtcNow.ToString("yyyy-MM-dd HH:mm:ss") } },
                    { "ttl", new AttributeValue { N = ((Int32)(DateTime.UtcNow.AddDays(30).Subtract(new DateTime(1970, 1, 1))).TotalSeconds).ToString() } },
                };
                urls.ToList().ForEach(x => row[x.Key] = new AttributeValue { B = ToGzipMemoryStream(x.Value) });
                dynamoClient.PutItem(TableName, row);
                // dynamoClient.UpdateItem(TableName, row, null); 
                // ^ Not working yet, if we decide not to do batch PutItem, then UpdateItem will work well for multi-content ad templates. Ideally we batch though.
                return true;
            }
            catch (Exception x)
            {
                Log.Error("Error saving click url for emailId " + emailId  + ": " + x);
                return false;
            }
        }

        private static MemoryStream ToGzipMemoryStream(string value)
        {
            MemoryStream output = new MemoryStream();
            using (GZipStream zipStream = new GZipStream(output, CompressionMode.Compress, true))
            using (StreamWriter writer = new StreamWriter(zipStream))
            {
                writer.Write(value);
            }
            return output;
        }

        private static string FromGzipMemoryStream(MemoryStream stream)
        {
            using (GZipStream zipStream = new GZipStream(stream, CompressionMode.Decompress))
            using (StreamReader reader = new StreamReader(zipStream))
            {
                return reader.ReadToEnd();
            }
        }
    }
}
