﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization.Json;
using System.Text;
using Amazon;
using Amazon.DynamoDBv2;
using Amazon.DynamoDBv2.DataModel;
using Amazon.DynamoDBv2.DocumentModel;
using Serilog;

namespace Mobile.Search.Web.Helpers
{
    public static class DynamoHelper
    {

        private static readonly ILogger Log = Serilog.Log.ForContext(typeof (DynamoHelper));

        private static IAmazonDynamoDB _dynamoClient = null;
        private static DynamoDBContext _context;
        public static string AdTableName = "mobtrack_ad_config";
        public static string ClickTableName = "mobtrack_clicks";
        public static string ConversionTableName = "mobtrack_conversions";
        public static string ConversionProcessedTableName = "mobtrack_conversions_processed";
        private static object init = new object();


        static DynamoHelper()
        {
            lock (init)
            {
                if (_dynamoClient == null)
                {
                    _dynamoClient = AWSClientFactory.CreateAmazonDynamoDBClient();
                }
                if (_context == null)
                {
                    _context = new DynamoDBContext(_dynamoClient);
                }

            }
        }


        //public static TrackerAd GetAd(int adId)
        //{

        //    return Query<TrackerAd>(adId).FirstOrDefault();

        //}



        public static T Load<T>(object hashkey)
        {
            return _context.Load<T>(hashkey);
        }

        public static List<T> Query<T>(object hashkey)
        {
            return new List<T>(_context.Query<T>(hashkey));
        }

        public static List<T> Query<T>(object hashkey, object rangekey)
        {
            return new List<T>(_context.Query<T>(hashkey, QueryOperator.Equal, rangekey));
        }

        public static List<T> Scan<T>(params ScanCondition[] conditions)
        {
            return new List<T>(_context.Scan<T>(conditions));
        }


        public static T Converter<T>(Document doc) where T : new()
        {
            var obj = new T();

            var attributes = doc.GetAttributeNames();


            //set anything we can find that matches
            var tadType = typeof (T);
            foreach (var attr in attributes)
            {

                var prop = tadType.GetProperty(attr);
                if (prop != null)
                {
                    if (prop.PropertyType.GetMethod("Parse", new[] {typeof (string)}) != null)
                    {
                        if (prop.PropertyType == typeof (bool))
                        {
                            prop.SetValue(obj, doc[attr] == "1");
                        }
                        else
                        {
                            prop.SetValue(obj,
                                prop.PropertyType.GetMethod("Parse", new[] {typeof (string)})
                                    .Invoke(obj, new object[] {(string) doc[attr]}));
                        }

                    }
                    else if (prop.PropertyType == typeof (string))
                    {
                        prop.SetValue(obj, (string) doc[attr]);
                    }


                }

            }
            return obj;
        }

        public class Key
        {
            public Key()
            {
            }

            public Key(string name, DynamoDBEntry value)
            {
                Name = name;
                Value = value;
            }

            public string Name { get; set; }
            public DynamoDBEntry Value { get; set; }
        }

        public static Document Converter<T>(T obj, params Key[] keys)
        {

            var doc = new Document();

            if (keys != null)
            {
                foreach (Key key in keys)
                {
                    if (!string.IsNullOrEmpty(key.Name) && key.Value != null)
                    {
                        doc[key.Name] = key.Value;
                    }
                }
            }


            foreach (var prop in typeof (T).GetProperties())
            {

                dynamic value = prop.GetValue(obj);
                doc[prop.Name] = value;



            }

            return doc;
        }

        public class DictonaryConverter<T, U> : IPropertyConverter
        {


            public static string ToJson(Dictionary<T, U> data)
            {
                if (data == null)
                {
                    return null;
                    throw new ArgumentNullException("data");

                }

                var serializer = new DataContractJsonSerializer(typeof (Dictionary<T, U>),
                    new DataContractJsonSerializerSettings() {UseSimpleDictionaryFormat = true});

                var dict = new Dictionary<T, U>(data);

                using (var stream = new MemoryStream())
                {
                    serializer.WriteObject(stream, dict);
                    var serialized = Encoding.UTF8.GetString(stream.ToArray());
                    return serialized;

                }

            }



            public DynamoDBEntry ToEntry(object value)
            {

                if (value == null)
                {
                    throw new ArgumentNullException("value");

                }

                var data = value as Dictionary<T, U>;



                if (data == null)
                {
                    throw new ArgumentException("Value was not a valid dictionary of type Dictionary<" + typeof (T).Name +
                                                "," + typeof (U).Name + ">.");
                }
                var serializer = new DataContractJsonSerializer(typeof (Dictionary<T, U>),
                    new DataContractJsonSerializerSettings() {UseSimpleDictionaryFormat = true});

                var dict = new Dictionary<T, U>(data);

                using (var stream = new MemoryStream())
                {
                    serializer.WriteObject(stream, dict);
                    var serialized = Encoding.UTF8.GetString(stream.ToArray());
                    DynamoDBEntry entry = serialized;
                    return entry;

                }

            }

            public object FromEntry(DynamoDBEntry entry)
            {
                if (entry == null)
                {
                    throw new ArgumentNullException("entry");

                }

                var serializer = new DataContractJsonSerializer(typeof (Dictionary<T, U>),
                    new DataContractJsonSerializerSettings() {UseSimpleDictionaryFormat = true});

                using (var stream = new MemoryStream(Encoding.UTF8.GetBytes(entry)))
                {

                    var obj = serializer.ReadObject(stream);
                    var dict = (Dictionary<T, U>) obj;

                    return dict;
                }
            }
        }
    }
}