﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Linq;
using System.Security.Claims;
using System.Security.Principal;
using System.Text.RegularExpressions;
using System.Web;

namespace Mobile.Search.Web.Helpers
{
    public class RequestHelper
    {

        private const string TenantIdClaimType = "http://schemas.microsoft.com/identity/claims/tenantid";
        private static readonly string clientId = ConfigurationManager.AppSettings["ida:ClientId"];
        private static readonly string appKey = ConfigurationManager.AppSettings["ida:AppKey"];
        private static readonly string graphResourceId = ConfigurationManager.AppSettings["ida:GraphUrl"];

        private static readonly string graphUserUrl = "https://graph.windows.net/{0}/me?api-version=" +
                                               ConfigurationManager.AppSettings["ida:GraphApiVersion"];

        public static string GetUserName(IPrincipal user)
        {
            string name = null;

            if (user != null && user.Identity.IsAuthenticated)
            {
                name = user.Identity.Name;
                try
                {
                    var cp = ClaimsPrincipal.Current;
                    var fname = cp.FindFirst(ClaimTypes.GivenName).Value;
                    var lname = cp.FindFirst(ClaimTypes.Surname).Value;


                    if (!string.IsNullOrEmpty(fname) && !string.IsNullOrEmpty(lname))
                        switch (fname.ToLower())
                        {
                            case "chris":
                                {
                                    name = $"{fname} {lname.Substring(0, 1)}";
                                    break;
                                }
                            case "rebecca":
                                {
                                    if (string.Equals(lname, "percell", StringComparison.CurrentCultureIgnoreCase))
                                    {
                                        name = "Becky";
                                    }
                                    else
                                    {
                                        name = fname;
                                    }
                                    break;
                                }
                            case "matt":
                                {
                                    if (string.Equals(lname, "hoggatt", StringComparison.CurrentCultureIgnoreCase))
                                    {
                                        name = "Boss Hoggatt";
                                    }
                                    else
                                    {
                                        name = fname;
                                    }
                                    break;
                                }
                            default:
                                {
                                    name = fname;
                                    break;
                                }
                        }
                }
                catch (Exception)
                {

                }


            }

            return name;

        }
        public static bool IsMobile(string useragent)

        {
            if (!string.IsNullOrEmpty(useragent))
            {
                return Regex.Match(useragent, "andriod|iphone|ipad|ipod", RegexOptions.IgnoreCase).Success;

            }

            return false;
        }


        private static string GetXForwardedForIp(string iplist)
        {
            if (string.IsNullOrEmpty(iplist))
                return null;
            if (iplist.Contains(",") || iplist.Trim().Contains(' '))
            {
                return iplist.Trim().Split(new[] { ',', ' ' }).First().Trim();
            }
            return iplist;
        }

        public static string GetIp(HttpContextBase context)
        {
            var ip = "0.0.0.0";
            if (context != null)
            {
                ip = GetXForwardedForIp(HttpContext.Current.Request.Headers["x-forwarded-for"]) ??
                     ((HttpContext.Current.Request.UserHostAddress == null ||
                       HttpContext.Current.Request.UserHostAddress == "127.0.0.1" ||
                       HttpContext.Current.Request.UserHostAddress.StartsWith("192.168"))
                          ? "74.112.58.45"
                          : HttpContext.Current.Request.UserHostAddress);
            }
            return ip;
        }
    }

}