﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Timers;
using Mobile.Search.Web.Models;

namespace Mobile.Search.Web.Helpers
{
    public class SourceTagMappingHelper
    {
        private static readonly object LockObj = new object();
        private static List<SourceTagMapping> _config = new List<SourceTagMapping>();

        public string System { get; set; }
        private readonly Timer _refreshTimer = new Timer(new TimeSpan(0, 0, 5, 0).TotalMilliseconds);

        public DateTime LastRefresh { get; protected set; }

        private bool _autorefresh = true;
        public bool AutoRefresh
        {
            get { return _autorefresh; }
            set
            {
                if (_autorefresh && !value)
                {
                    _refreshTimer.Stop();
                }
                if (!_autorefresh && value)
                {
                    _refreshTimer.Start();
                }

                _autorefresh = value;
                _refreshTimer.AutoReset = _autorefresh;
            }

        }

        public List<SourceTagMapping> Mappings
        {
            get
            {
                if (_config.Count > 0) return _config;
                var task = RefreshMappings();
                task.Wait(100);

                return _config;
            }
        }

        private Task RefreshMappings()
        {
            return Task.Factory.StartNew(() =>
            {
                lock (LockObj)
                {
                    if (LastRefresh.AddMinutes(5) <= DateTime.UtcNow)
                    {
                        _config = DynamoHelper.Scan<SourceTagMapping>();
                        LastRefresh = DateTime.UtcNow;
                    }
                }
            });
        }

        public SourceTagMapping GetSourceTagMappings(long sourceTagId, string system)
        {
            SourceTagMapping map = null;
            Func<SourceTagMapping, bool> predicate =
                x => x.SourceTagMapId == sourceTagId && system.ToLower() == x.System.ToLower();
            if (Mappings != null && Mappings.Any(predicate))
            {
                map = Mappings.FirstOrDefault(predicate);
            }
            else
            {
                // refresh and force a get from dynamo
                RefreshMappings();
                return DynamoHelper.Query<SourceTagMapping>(sourceTagId, system).FirstOrDefault();
            }

            return map;
        }

        private static void OnTimedEvent(object source, ElapsedEventArgs e)
        {
            Instance.RefreshMappings();
        }


        private static SourceTagMappingHelper _instance;

        private SourceTagMappingHelper()
        {
            _refreshTimer.Enabled = true;
            _refreshTimer.AutoReset = true;
            _refreshTimer.Start();
            _refreshTimer.Elapsed += OnTimedEvent;
        }

        public static SourceTagMappingHelper Instance
        {
            get { return _instance ?? (_instance = new SourceTagMappingHelper()); }
        }
        /// <summary>
        /// Moved this method to the Mobile.Search.Shared Project in the YahooSourceTagHelper class    --Urvil Shah(01/16/2018)
        /// </summary>
        /// <param name="productId"></param>
        /// <param name="sourceTagOverride"></param>
        /// <returns></returns>
        //public static string GetSourceTag(int? productId, string sourceTagOverride)
        //{
        //    //default is 2click other-non-google
        //    var sourceTag = "adknowledge_mobile_2click_j_search";

        //    //1click
        //    if (!string.IsNullOrEmpty(sourceTagOverride) && sourceTagOverride.ToLower().Contains("1click"))
        //    {
        //        //default is 1click other-non-google
        //        sourceTag = "adknowledge_mobile_1click_j_search";
        //        switch (productId)
        //        {

        //            case 100120: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100155: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100168: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100173: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100187: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100213: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100214: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100264: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100283: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100295: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100331: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100333: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100339: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100344: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100350: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100351: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100352: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100357: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100369: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100373: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100397: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100407: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100411: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100428: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100433: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100459: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100460: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100461: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100462: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100466: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100469: { sourceTag = "adknowledge_mobile_1click_e_search"; break; } //coupon-non-google
        //            case 100146: { sourceTag = "adknowledge_mobile_1click_search"; break; } //senior-discounts-google
        //            case 100164: { sourceTag = "adknowledge_mobile_1click_search"; break; } //senior-discounts-google
        //            case 100201: { sourceTag = "adknowledge_mobile_1click_search"; break; } //senior-discounts-google
        //            case 100268: { sourceTag = "adknowledge_mobile_1click_search"; break; } //senior-discounts-google
        //            case 100279: { sourceTag = "adknowledge_mobile_1click_search"; break; } //senior-discounts-google
        //            case 100321: { sourceTag = "adknowledge_mobile_1click_search"; break; } //senior-discounts-google
        //            case 100438: { sourceTag = "adknowledge_mobile_1click_search"; break; } //senior-discounts-google
        //                                                                                    // case 100146: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100162: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //                                                                                      // case 100164: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100191: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100249: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100253: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100265: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //                                                                                      // case 100268: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100272: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100276: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100278: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100297: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100313: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100316: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100332: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100340: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100348: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100430: { sourceTag = "adknowledge_mobile_1click_k_search"; break; } //senior-discounts-non-google
        //            case 100023: { sourceTag = "adknowledge_mobile_1click_d_search"; break; } //weather-google
        //            case 100154: { sourceTag = "adknowledge_mobile_1click_d_search"; break; } //weather-google
        //            case 100163: { sourceTag = "adknowledge_mobile_1click_d_search"; break; } //weather-google
        //            case 100240: { sourceTag = "adknowledge_mobile_1click_d_search"; break; } //weather-google
        //            case 100273: { sourceTag = "adknowledge_mobile_1click_d_search"; break; } //weather-google
        //            case 100289: { sourceTag = "adknowledge_mobile_1click_d_search"; break; } //weather-google
        //            case 100309: { sourceTag = "adknowledge_mobile_1click_d_search"; break; } //weather-google
        //            case 100375: { sourceTag = "adknowledge_mobile_1click_d_search"; break; } //weather-google
        //            case 100400: { sourceTag = "adknowledge_mobile_1click_d_search"; break; } //weather-google
        //            case 100424: { sourceTag = "adknowledge_mobile_1click_d_search"; break; } //weather-google
        //            case 100242: { sourceTag = "adknowledge_mobile_1click_c_search"; break; } //weather-red-alerts-google
        //            case 100291: { sourceTag = "adknowledge_mobile_1click_c_search"; break; } //weather-red-alerts-google
        //            case 100293: { sourceTag = "adknowledge_mobile_1click_c_search"; break; } //weather-red-alerts-google
        //            case 100335: { sourceTag = "adknowledge_mobile_1click_c_search"; break; } //weather-red-alerts-google
        //            case 100365: { sourceTag = "adknowledge_mobile_1click_c_search"; break; } //weather-red-alerts-google
        //            case 100368: { sourceTag = "adknowledge_mobile_1click_c_search"; break; } //weather-red-alerts-google
        //            case 100374: { sourceTag = "adknowledge_mobile_1click_c_search"; break; } //weather-red-alerts-google

        //                //other


        //            case 100197: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100018: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100156: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100171: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100206: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100298: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100305: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100320: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100363: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100381: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100422: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100437: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100130: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100323: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100406: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            //case 100120: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            //case 100155: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            //case 100168: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            //case 100170: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            //case 100214: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100306: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100364: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            //case 100369: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100199: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100208: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100307: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100008: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100236: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100304: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100007: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100020: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100070: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100198: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100200: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100325: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100341: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100064: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100124: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100237: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100403: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100121: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100294: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100138: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100238: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100194: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100195: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100202: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100337: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100209: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100239: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100244: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100308: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100143: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100416: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100141: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google
        //            case 100248: { sourceTag = "adknowledge_mobile_1click_f_search"; break; } //other-google




        //        }
        //    }
        //    else
        //    {
        //        //2click
        //        switch (productId)
        //        {


        //            case 100120: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100155: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100168: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100173: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100187: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100213: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100214: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100264: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100283: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100295: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100331: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100333: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100339: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100344: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100350: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100351: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100352: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100357: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100369: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100373: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100397: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100407: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100411: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100428: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100433: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100459: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100460: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100461: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100462: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100466: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100469: { sourceTag = "reachmobi_mobileinbox_e_search"; break; } //coupon-non-google
        //            case 100146: { sourceTag = "reachmobi_mobileinbox_a_search"; break; } //senior-discounts-google
        //            case 100164: { sourceTag = "reachmobi_mobileinbox_a_search"; break; } //senior-discounts-google
        //            case 100201: { sourceTag = "reachmobi_mobileinbox_a_search"; break; } //senior-discounts-google
        //            case 100268: { sourceTag = "reachmobi_mobileinbox_a_search"; break; } //senior-discounts-google
        //            case 100279: { sourceTag = "reachmobi_mobileinbox_a_search"; break; } //senior-discounts-google
        //            case 100321: { sourceTag = "reachmobi_mobileinbox_a_search"; break; } //senior-discounts-google
        //            case 100438: { sourceTag = "reachmobi_mobileinbox_a_search"; break; } //senior-discounts-google
        //         // case 100146: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100162: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //         // case 100164: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100191: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100249: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100253: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100265: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //         // case 100268: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100272: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100276: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100278: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100297: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100313: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100316: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100332: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100340: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100348: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100430: { sourceTag = "reachmobi_mobileinbox_b_search"; break; } //senior-discounts-non-google
        //            case 100023: { sourceTag = "reachmobi_mobileinbox_d_search"; break; } //weather-google
        //            case 100154: { sourceTag = "reachmobi_mobileinbox_d_search"; break; } //weather-google
        //            case 100163: { sourceTag = "reachmobi_mobileinbox_d_search"; break; } //weather-google
        //            case 100240: { sourceTag = "reachmobi_mobileinbox_d_search"; break; } //weather-google
        //            case 100273: { sourceTag = "reachmobi_mobileinbox_d_search"; break; } //weather-google
        //            case 100289: { sourceTag = "reachmobi_mobileinbox_d_search"; break; } //weather-google
        //            case 100309: { sourceTag = "reachmobi_mobileinbox_d_search"; break; } //weather-google
        //            case 100375: { sourceTag = "reachmobi_mobileinbox_d_search"; break; } //weather-google
        //            case 100400: { sourceTag = "reachmobi_mobileinbox_d_search"; break; } //weather-google
        //            case 100424: { sourceTag = "reachmobi_mobileinbox_d_search"; break; } //weather-google
        //            case 100242: { sourceTag = "reachmobi_mobileinbox_c_search"; break; } //weather-red-alerts-google
        //            case 100291: { sourceTag = "reachmobi_mobileinbox_c_search"; break; } //weather-red-alerts-google
        //            case 100293: { sourceTag = "reachmobi_mobileinbox_c_search"; break; } //weather-red-alerts-google
        //            case 100335: { sourceTag = "reachmobi_mobileinbox_c_search"; break; } //weather-red-alerts-google
        //            case 100365: { sourceTag = "reachmobi_mobileinbox_c_search"; break; } //weather-red-alerts-google
        //            case 100368: { sourceTag = "reachmobi_mobileinbox_c_search"; break; } //weather-red-alerts-google
        //            case 100374: { sourceTag = "reachmobi_mobileinbox_c_search"; break; } //weather-red-alerts-google



        //            case 100197: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100018: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100156: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100171: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100206: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100298: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100305: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100320: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100363: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100381: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100422: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100437: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100130: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100323: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100406: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            //case 100120: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            //case 100155: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            //case 100168: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            //case 100170: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            //case 100214: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100306: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100364: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            //case 100369: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100199: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100208: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100307: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100008: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100236: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100304: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100007: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100020: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100070: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100198: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100200: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100325: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100341: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100064: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100124: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100237: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100403: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100121: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100294: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100138: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100238: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100194: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100195: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100202: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100337: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100209: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100239: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100244: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100308: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100143: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100416: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100141: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google
        //            case 100248: { sourceTag = "adknowledge_mobile_2click_f_search"; break; } //other-google





        //        }

        //    }


        //    return sourceTag;
        //}
    }
}