﻿using System;
using System.Collections.Generic;
using Amazon;
using Amazon.Runtime;
using Chronos.AWS;
using Chronos.Configuration;
using Newtonsoft.Json.Linq;
using Serilog;

namespace Mobile.Search.Web.Helpers
{
    public class TemplateCache
    {
        private static readonly string AWSAccessKey = ConfigUtilities.GetAppSetting("AWSAccessKey");
        private static readonly string AWSSecretKey = ConfigUtilities.GetAppSetting("AWSSecretKey");
        private readonly Dictionary<string, TemplateCacheObj> _templates;
        private TemplateCache()
        {
           _templates = new Dictionary<string, TemplateCacheObj>(); 
        }

        public static TemplateCache Instance = new TemplateCache();

        public bool TryGetTemplateHtml(string templateKey, out string html)
        {
            // templates dict has the entry
            // the entry is not null, and the html prop is not null
            // it hasn't been in cache longer than 5 minuts
            if (_templates.ContainsKey(templateKey) && !string.IsNullOrEmpty(_templates[templateKey]?.Html) && (DateTime.UtcNow - _templates[templateKey]?.CachedAt) < TimeSpan.FromSeconds(3))
            {
                html = _templates[templateKey].Html;
                return true;
            }

            if (TryGetTemplateHtmlFromDynamo(templateKey, out html))
            {
                _templates[templateKey] = new TemplateCacheObj
                {
                    CachedAt = DateTime.UtcNow,
                    Html = html,
                    TemplateKey = templateKey
                };
                return true;
            }
            else
            {
                Log.Error("Couldn't find template in DynamoDb {templateKey}", templateKey);
                 html = string.Empty;
                return false;               
            }


        }

        private bool TryGetTemplateHtmlFromDynamo(string templateKey, out string html)
        {
            var dynamoClient = new Dynamo(new BasicAWSCredentials(AWSAccessKey, AWSSecretKey), RegionEndpoint.USWest2);
            var json = dynamoClient.Get("prd_2click_templates", "templateKey", templateKey);
            if (string.IsNullOrEmpty(json))
            {
                html = string.Empty;
                return false;
            }
            var jsonObject = JObject.Parse(json);
            html = (string) jsonObject["html"];
            if (string.IsNullOrEmpty(html))
            {
                Log.Error("Couldn't find template with key {key}", templateKey);
                html = string.Empty;
                return false;
            }
            return true;
        }
    }

    public class TemplateCacheObj
    {
        public string TemplateKey { get; set; }
        public string Html { get; set; }
        public DateTime CachedAt { get; set; }
    }
}