﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Reactive.Linq;
using Chronos.Dapper.Chronos.Dapper;
using Npgsql;
using Serilog;

namespace Mobile.Search.Web.Helpers
{
    public class TopKeywordsCache
    {
        private static readonly string TopKeywordsConnStr =
            ConfigurationManager.ConnectionStrings["TopKeywordsRedshiftConnection"].ConnectionString;

        private TopKeywordsCache()
        {
            RefreshKeywords();
            Observable.Interval(TimeSpan.FromMinutes(30))
                .Subscribe(x => RefreshKeywords());
        }
        public void RefreshKeywords()
        {
            using (var conn = new NpgsqlConnection(TopKeywordsConnStr))
            {
                try
                {
                    _topKeywords =
                        conn.Query<string>("select keyword from x_category_score order by cpm desc limit 15").ToList();
                }
                catch (Exception x)
                {
                    Log.Error(x, "Couldn't get top keywords");
                }
            }
        }


        private List<string> _topKeywords;

        public List<string> TopKeywords
        {
            get { return _topKeywords.ToList(); }
        }

        public static TopKeywordsCache Instance = new TopKeywordsCache();
    }
}